@@ -51,7 +51,7 @@ typedef enum {
 	GNOME_CLIENT_STATE_COLLECT_INFO,
 	GNOME_CLIENT_STATE_REQUEST_ACCESS,
 	GNOME_CLIENT_STATE_EXECUTE_OP,
-	GNOME_CLIENT_STATE_WRITE_REPLY,
+	GNOME_CLIENT_STATE_WRITE_REPLY
 } GnomeKeyringClientStates;
 
 typedef struct {
@@ -156,7 +156,7 @@ read_unix_socket_credentials (int fd,
 	{
 #ifdef SO_PEERCRED
 		struct ucred cr;   
-		int cr_len = sizeof (cr);
+		socklen_t cr_len = sizeof (cr);
 		
 		if (getsockopt (fd, SOL_SOCKET, SO_PEERCRED, &cr, &cr_len) == 0 &&
 		    cr_len == sizeof (cr)) {
