@@ -565,7 +565,7 @@ access_request_for_new_keyring_password (const char *keyring_name)
 }
 
 static GnomeKeyringAccessRequest *
-access_request_default_keyring ()
+access_request_default_keyring (void)
 {
 	GnomeKeyringAccessRequest *access_request;
 	access_request = g_new0 (GnomeKeyringAccessRequest, 1);
@@ -797,6 +797,39 @@ op_list_keyrings_execute (GString *packet,
 }
 
 
+static gboolean
+op_set_keyring_info_execute (GString *packet,
+			     GString *result,
+			     GnomeKeyringApplicationRef *app_ref,
+			     GList *access_requests)
+{
+	char    *keyring_name;
+	gboolean lock_on_idle;
+	guint32  lock_timeout;
+	GnomeKeyring *keyring;
+	
+	if (!gnome_keyring_proto_decode_set_keyring_info (packet,
+							  &keyring_name,
+							  &lock_on_idle,
+							  &lock_timeout)) {
+		return FALSE;
+	}
+	
+	keyring = find_keyring (keyring_name);
+	if (keyring == NULL) {
+		gnome_keyring_proto_add_uint32 (result, GNOME_KEYRING_RESULT_NO_SUCH_KEYRING);
+	} else {
+		gnome_keyring_proto_add_uint32 (result, GNOME_KEYRING_RESULT_OK);
+		
+		keyring->lock_on_idle = lock_on_idle;
+		keyring->lock_timeout = lock_timeout;
+	}
+	
+	g_free (keyring_name);
+
+	return TRUE;
+}
+
 static gboolean
 op_get_keyring_info_execute (GString *packet,
 			     GString *result,
@@ -2387,7 +2420,7 @@ GnomeKeyringOperationImplementation keyring_ops[] = {
 	{ NULL, op_unlock_keyring_execute}, /* UNLOCK_KEYRING */
 	{ NULL, op_delete_keyring_execute}, /* DELETE_KEYRING */
 	{ NULL, op_get_keyring_info_execute}, /* GET_KEYRING_INFO */
-	{ NULL, NULL}, /* SET_KEYRING_INFO */
+	{ NULL, op_set_keyring_info_execute}, /* SET_KEYRING_INFO */
 	{ op_list_items_collect, op_list_items_execute}, /* LIST_ITEMS */
 	{ op_find_collect, op_find_execute }, /* FIND */
 	{ op_create_item_collect, op_create_item_execute}, /* CREATE_ITEM */
@@ -2396,7 +2429,7 @@ GnomeKeyringOperationImplementation keyring_ops[] = {
 	{ op_set_item_info_or_attributes_collect, op_set_item_info_execute}, /* SET_ITEM_INFO */
 	{ op_get_item_info_or_attributes_collect, op_get_item_attributes_execute}, /* GET_ITEM_ATTRIBUTES */
 	{ op_set_item_info_or_attributes_collect, op_set_item_attributes_execute}, /* SET_ITEM_ATTRIBUTES */
-	{ op_set_item_info_or_attributes_collect, op_get_item_acl_execute}, /* GET_ITEM_ACL */
+	{ op_get_item_info_or_attributes_collect, op_get_item_acl_execute}, /* GET_ITEM_ACL */
 	{ op_set_item_info_or_attributes_collect, op_set_item_acl_execute}, /* SET_ITEM_ACL */
 };
 
@@ -2436,16 +2469,27 @@ close_stdinout (void)
 	close (fd);
 }
 
+static gboolean
+lifetime_slave_pipe_io (GIOChannel  *channel,
+			GIOCondition cond,
+			gpointer     callback_data)
+{
+        cleanup_socket_dir ();
+        _exit (2);
+}
+
 int
 main (int argc, char *argv[])
 {
 	const char *path;
+	char *fd_str;
+	int fd;
 	pid_t pid;
 	gboolean foreground;
 	gboolean daemon;
+	GIOChannel *channel;
 	int i;
 	
-	
 	if (!create_master_socket (&path)) {
 		exit (1);
 	}
@@ -2536,6 +2580,21 @@ main (int argc, char *argv[])
 	update_keyrings_from_disk ();
 
 	loop = g_main_loop_new (NULL, FALSE);
+
+	fd_str = getenv ("GNOME_KEYRING_LIFETIME_FD");
+	if (fd_str != NULL && fd_str[0] != 0) {
+		fd = atoi (fd_str);
+		if (fd != 0) {
+			channel = g_io_channel_unix_new (fd);
+			g_io_add_watch (channel,
+					G_IO_IN | G_IO_HUP,
+					lifetime_slave_pipe_io, NULL);
+			g_io_channel_unref (channel);
+		}
+		
+	}
+	
+	
 	g_main_loop_run (loop);
 	
 	cleanup_socket_dir ();
